import os
import json
from openpyxl import Workbook, load_workbook
from pyboon import file as f
from rich.console import Console


def csv2xl(csvfile):
    wb = Workbook()
    ws = wb.active

    txt = f.read(csvfile)

    for line in txt.split("\n"):
        print(line)
        ws.append(line.split(","))

    wb.save(csvfile.replace(".txt", ".xlsx"))


def xl2csv(xlfile):
    wb = load_workbook(xlfile)
    ws = wb[wb.sheetnames[0]]

    lines = []
    for row in ws.iter_rows():
        values = [cell.value for cell in row]
        if values[0]:
            lines.append(",".join(values))

    f.write(os.path.join(os.getcwd(), xlfile.replace(
        ".xlsx", ".txt")), "\n".join(lines))


def to_json(d):
    return json.dumps(d, ensure_ascii=False)


def to_json_array(l):
    return [to_json(x) for x in l]


def to_md_line(d, head=[]):
    if not head:
        head = [k for k,_ in d.items()]
    return "| "+" | ".join([str(d[k]) for k in head])+" |"

def get_show_list(d, head=[], show={}):
    if not head:
        head = [k for k, _ in d.items()]
    if not show:
        show_list = head
    else:
        show_list = [show[k] for k in head ]
    return show_list

def to_md(l, head=[], show={}):
    if not l or not l[0]:
        return ""
    show_list = get_show_list(l[0],head,show)
    showline = "| "+" | ".join(show_list)+" |"
    splitline = "| "+" |".join([":---:" for k, _ in l[0].items()])+" |"
    lines = [showline, splitline]
    for row in l:
        line = to_md_line(row, head)
        lines.append(line)
    return "\n".join(lines)


def to_html(l,head=[], show={}):
    md = to_md(l,head,show)
    import markdown
    html = markdown.markdown(md, extensions=['tables'])
    from bs4 import BeautifulSoup
    soup = BeautifulSoup(html, 'html.parser')
    return soup.prettify()


def to_csv_line(d, head=[]):
    if not head:
        head = [k for k,_ in d.items()]
    return ",".join([str(d[k]) for k in head])


def to_csv(l, head=[], show=[]):
    if not l or not l[0]:
        return ""
    show_list = get_show_list(l[0],head,show)
    showline = ",".join(show_list)
    lines = [showline]
    for row in l:
        line = to_csv_line(row)
        lines.append(line)
    return "\n".join(lines)


def to_xlsx(l, head=[], show=[]):
    wb = Workbook()
    ws = wb.active

    for line in to_csv(l,head,show).split("\n"):
        print(line)
        ws.append(line.split(","))

    wb.save("a.xlsx")


if __name__ == '__main__':
    console = Console()
    console.print(to_csv([{"a": 1, "b": "c"}, {"a": 2, "b": "c"}],head=['b','a']))
    console.print(to_xlsx([{"a": 1, "b": "c"}, {"a": 2, "b": "c"}],head=['b','a']))
    console.print(to_md([{"a": 1, "b": "c"}, {"a": 2, "b": "c"}],head=['b','a']))
    console.print(to_html([{"a": 1, "b": "c"}, {"a": 2, "b": "c"}],head=['b','a'],show={"a":"第一","b":"第二",}))



import json


def stringify(d):
    return json.dumps(d, ensure_ascii=False)


def prettify(d):
    return json.dumps(d, indent=2, ensure_ascii=False)


def parse(json_str):
    d = json.loads(json_str)
    return d




from pyboon import x, f

import pymysql
import dataset
import re
import os
from rich.table import Table
from rich.console import Console
console = Console()

pymysql.install_as_MySQLdb()

f.CURRENT_DIR = os.path.split(os.path.realpath(__file__))[0]
DB_URL = 'mysql://root:aFM4JrZIDgpGiqJn@localhost:3306/hello?charset=UTF8MB4'

def get_db(url):
    return dataset.connect(url, engine_kwargs={'encoding': 'utf-8'})


def info(schema="hello"):
    db = get_db(DB_URL)
    tables = db.query(f'''
SELECT 
table_schema as `schema`,
table_name as name, 
table_comment as comment
FROM information_schema.tables
WHERE table_schema = '{schema}'
ORDER BY table_name DESC;
    ''')
    tables = list(tables)
    for table in tables:
        create_table_sql = db.query(
            f"show create table {schema}.{table['name']};").next(
        )['Create Table']
        console.print(table)
        console.print(create_table_sql)
        table['sql'] = create_table_sql
        columns = db.query(f'''
Select 
COLUMN_NAME as name, 
DATA_TYPE as dtype, 
character_maximum_length as length,
is_nullable as nullable, 
COLUMN_DEFAULT as value, 
COLUMN_KEY as keytype,
extra,
ordinal_position as idx, 
COLUMN_COMMENT as comment
from INFORMATION_SCHEMA.COLUMNS
Where table_name = '{table['name']}'
AND table_schema = '{schema}'
    ''')
        columns = list(columns)
        for column in columns:
            column['table'] = table['name']
            column['schema'] = schema
        table['columns'] = columns
        # console.print(json.dumps(table, ensure_ascii=False))
    return tables


def p():
    tables = info()
    for table in tables:
        columns = table['columns']
        # console.print(table,"green")
        ctable = Table()
        for k, v in columns[0].items():
            ctable.add_column(f'[red]{k}')
        for column in columns:
            ctable.add_row(*[f'[yellow]{v}' for k, v in column.items()])
        console.print(ctable)


def t(comment, table, schema='hello'):
    db = get_db(DB_URL)
    return db.query(f"alter table {schema}.{table} comment '{comment}';")


def c(comment, column, table="hello_holiday", schema='hello'):
    db = get_db(DB_URL)
    coltype = db.query(f'''
Select 
COLUMN_NAME as name, 
COLUMN_TYPE as coltype
from INFORMATION_SCHEMA.COLUMNS
Where table_name = '{table}'
AND table_schema = '{schema}'
AND COLUMN_NAME = '{column}'
    ''').next()['coltype']
    return db.query(f"ALTER TABLE {schema}.{table} MODIFY {column} {coltype} COMMENT '{comment}';")


def todb():
    db = get_db(DB_URL)
    meta_column = db['meta_column']
    meta_column.insert_many([column for table in info()
                            for column in table['columns']])
    meta_table = db['meta_table']
    meta_table.insert_many(
        [dict(name=table['name'], comment=table['comment']) for table in info()])

def tm():
    models = []
    for table in info():
        model = dict(model=table['name'], Model=x.to_pascal(
            table['name']), modelCN=table['comment'])
        model.setdefault("fields", [])
        # print(model)
        for column in table['columns']:
            field = dict(
                name=column['name'], type=column['dtype'], comment=column['comment'],pk=column['name']=='id')
            # print(field)
            model['fields'].append(field)
        models.append(model)
    return models    
    
def qb(l, sql):
    ret = re.search(r"SELECT (.*) FROM", sql)
    _, v = ret.group(0), ret.group(1)
    fields = [field.strip() for field in v.split(",")]
    print(fields)
    result = []
    for item in l:
        n = {}
        for field in fields:
            if field in item:
                n[field] = item[field]
            else:
                print(field, "not in item")
        result.append(n)
    print(result)
    return result


def qb_one(l, sql):
    ret = re.search(r"SELECT (.*) FROM", sql)
    _, v = ret.group(0), ret.group(1)
    fields = [field.strip() for field in v.split(",")]
    print(fields)
    result = []
    for item in l:
        n = {}
        for field in fields:
            if field in item:
                n[field] = item[field]
            else:
                print(field, "not in item")
        result.append(n)
    print(result)
    return result[0] if result else None


if __name__ == '__main__':
    qb([{"a": 1, "b": 2, "c": 3, "d": 4, }], "SELECT a,b FROM tab")
    info()
    p()
    tm()
    t('表', "meta_table", schema='hello')
    c('名称', 'name', table="meta_table", schema='hello')
    c('注释', 'comment', table="meta_table", schema='hello')
    # todb()


import datetime
import json
import time
import yaml
import re


def to_int(p):
    return int(p)


def to_str(p):
    return str(p)


def to_float(p):
    return float(p)


def to_datetime(s):
    return datetime.datetime.strptime(s, "%Y-%m-%d %H:%M:%S")


def to_datetime_str(dt):
    return time.strftime("%Y-%m-%d %H:%M:%S", dt.timetuple())


def to_yaml(d):
    content = yaml.dump(d, default_flow_style=False)
    content = re.sub(r'(\\u[a-zA-Z0-9]{4})', lambda x: x.group(
        1).encode("utf-8").decode("unicode-escape"), content)  # 逐段解码
    return content


def to_json(d):
    return json.dumps(d, ensure_ascii=False)


def to_json2(d):
    return json.dumps(d, ensure_ascii=False, indent=2)


def to_dict(s):
    # return json.loads(s)
    return yaml.load(s, Loader=yaml.FullLoader)



import json
from rich.table import Table
from rich.panel import Panel
from rich.syntax import Syntax
from rich.console import Console
console = Console()


def highlight(code, language="python"):
    if type(code) is dict or type(code) is list:
        code = json.dumps(code, indent=2, ensure_ascii=False)
    syntax = Syntax(code, language)
    console = Console()
    console.print(syntax)


def table(dicts):
    ctable = Table()
    for k, v in dicts[0].items():
        ctable.add_column(f'[red]{k}')
    for column in dicts:
        ctable.add_row(*[f'[yellow]{v}' for k, v in column.items()])
    console.print(ctable)
